package thesis.simulation;

import java.util.ArrayList;
import java.util.List;

import thesis.tree.Body;
import thesis.tree.TreeNode;
import thesis.tree.Vector;

public class Integrator {
  
  protected float stepSize;
  protected float openingAngle;
  protected float softeningFactor;
  
  public Integrator(float timeStep, float openingAngle, float softeningFactor) {
    this.stepSize = timeStep;
    this.openingAngle = openingAngle;
    this.softeningFactor = softeningFactor;
  }

  public void integrate(Body body) {
    body.setPosition(body.getPosition().add(
        body.getVelocity().multiply(stepSize)));
    
    body.setVelocity(body.getVelocity().add(
        body.getAcceleration().multiply(stepSize)));
  }
  
  public void calculateAcceleration(Body body, TreeNode root)
      throws InterruptedException {
    
    List<TreeNode> nodes = new ArrayList<TreeNode>();
    nodes.add(root);
    Vector totalForce = new Vector();
    
    while (!nodes.isEmpty()) {
      if (Thread.interrupted()) {
        throw new InterruptedException();
      }
      
      TreeNode node = nodes.remove(nodes.size() - 1);
      Vector displacement = node.getCenterOfMass().subtract(body.getPosition());
      
      boolean isWellSeparated = node.isLeaf() ||
          node.getSize() / displacement.length() < openingAngle;
      
      if (isWellSeparated) {
        Vector force =
          displacement.multiply(node.getTotalMass() /
              (float)Math.pow(
                  Math.pow(displacement.length(), 2) + softeningFactor,
              1.5)
          );
        totalForce = totalForce.add(force);
      } else {
        for (TreeNode child : node.getChildren()) {
          if (child != null) {
            nodes.add(child);
          }
        }
      }
    }
    
    body.setAcceleration(totalForce.divide(body.getMass()));
  }

}
